#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <errno.h>

#define	MAX_BUF_LEN		256


int set_speed(int fd, int speed)
{
	int i;
	int status;
	struct termios opt;

	status = tcgetattr(fd ,&opt);
	if(status != 0)
	{
		perror("get attr failed!\n");
		return -1;
	}

	tcflush(fd,TCIOFLUSH);
	cfsetispeed(&opt, speed);
	cfsetospeed(&opt, speed);

	status = tcsetattr(fd,TCSANOW,&opt);
	if(status != 0)
	{
		perror("set attr failed!\n");
		return -1;
	}

	tcflush(fd,TCIOFLUSH);

	return 0;
}

int set_com(int fd, int databits, int stopbits, int parity, int flowcontrol)
{
	struct termios opt;

	if( tcgetattr(fd ,&opt) != 0)
	{
		perror("get attr failed!\n");
		return -1;
	}

	opt.c_cflag &= ~CSIZE;
	switch(flowcontrol)
	{
		case 'n':
		case 'N':
			opt.c_iflag &= ~(IXON | IXOFF);
			opt.c_cflag &= ~IHFLOW;
			opt.c_cflag &= ~OHFLOW;
			break;
		case 's':
		case 'S':
			opt.c_cflag &= ~IHFLOW;
			opt.c_cflag &= ~OHFLOW;
			opt.c_iflag |= IXON;
			opt.c_iflag |= IXOFF;
			break;
		case 'h':
		case 'H':
			opt.c_iflag &= ~(IXON | IXOFF);
			opt.c_cflag |= IHFLOW;
			opt.c_cflag |= OHFLOW;
			break;
		default:
			printf("Unsupported flow control mode\n");
			return -1;
	}

	switch(databits)
	{
		case 5:
			opt.c_cflag |= CS5;
			break;
		case 6:
			opt.c_cflag |= CS6;
			break;
		case 7:
			opt.c_cflag |= CS7;
			break;
		case 8:
			opt.c_cflag |= CS8;
			break;
		default:
			printf("Unsupported data size\n");
			return -1;
	}

	switch(parity)
	{
		case 'n':
		case 'N':
			opt.c_cflag &= ~PARENB;
			opt.c_iflag &= ~INPCK;
			break;
		case 'o':
		case 'O':
			opt.c_cflag |= (PARODD|PARENB);
			opt.c_iflag |= INPCK;
			break;
		case 'e':
		case 'E':
			opt.c_cflag |= PARENB;
			opt.c_cflag &= ~PARODD;
			opt.c_iflag |= INPCK;
			break;
		default:
			printf("Unsupported parity\n");
			return -1;
	}

	switch(stopbits)
	{
		case 1:
			opt.c_cflag &= ~CSTOPB;
			break;
		case 2:
			opt.c_cflag |=  CSTOPB;
			break;
		default:
			printf("Unsupported stop bits\n");
			return -1;
	}

	if (parity != 'n')
	{
		opt.c_iflag |= INPCK;
		opt.c_cc[VTIME] = 150;
		opt.c_cc[VMIN] = 0;
	}
	tcflush(fd,TCIFLUSH);
	if(tcsetattr(fd,TCSANOW,&opt) != 0)
	{
		perror("set attr failed!\n");
		return -1;
	}

	return 0;
}

int OpenDev(char* Dev)
{
	int fd = open(Dev,O_RDWR);
	if( -1 == fd)
	{
		perror("open failed!\n");
		return -1;
	}
	else
		return fd;
}

void print_config(char *dev_name, int baud_rate, int databits, int stopbits, int parity,
					int flowcontrol)
{
	printf("Configurations:\n");
	printf("    device name  : %s\n" , dev_name    );
	printf("    baud rate    : %d\n", baud_rate   );
	printf("    data bits    : %d\n", databits    );
	printf("    stop bits    : %d\n", stopbits    );
	printf("    parity       : %c\n", parity      );
	printf("    flow control : %c\n" , flowcontrol );
}

void print_usage(int argc, char *argv[] )
{
	printf("usage:\n");
	printf("\t-D <device name>      Set device name,  \"/dev/ttyAP2\" as default value\n");
	printf("\t-b <baud rate>        Set baud rate,    9600 as default value\n");
	printf("\t-d <data bits>        Set data bits,    8 as default value\n");
	printf("\t-f <flow control>     Set flow control, \"none\" as default value\n");
	printf("\t-p <parity>           Set parity,       \"none\" as default value\n");
	printf("\t-s <stop bits>        Set stop bits,    1 as default value\n");
	printf("\t-h                    Print Help (this message) and exit\n");
	printf("\t-?                    Print Help (this message) and exit\n\n");

	printf("Example:\n");
	printf("\t%s\n", argv[0]);
	printf("\t%s -D /dev/ttyADV2 -b 9600 -d 8 -s 1 -p n -f n\n", argv[0]);
}

int main(int argc, char *argv[] )
{
	int fd;
	int nread, rx_count = 0;
	char buf[MAX_BUF_LEN], dev_name[MAX_BUF_LEN];
	int speed, databit, stopbit;
	char parity, flowcontrol;
	int  i, c;

	/* set default value */
	fd = -1;
	speed = 9600;
	databit = 8;
	stopbit = 1;
	parity = 'n';
	flowcontrol = 'n';

	while((c = getopt( argc, argv, "D:b:d:f:s:p:h?")) != -1)
	{
		switch( c ) {
		case 'D':
		{
			fd = OpenDev(optarg);
			strcpy(dev_name, optarg);
			break;
		}
		case 'b':
		{
			speed = atoi(optarg);
			break;
		}
		case 'd':
		{
			databit = atoi(optarg);
			break;
		}
		case 'f':
		{
			flowcontrol = *optarg;
			break;
		}
		case 's':
		{
			stopbit = atoi(optarg);
			break;
		}
		case 'p':
		{
			parity = *optarg;
			break;
		}
		case 'h':
		case '?':
		{
			/* check for 'help' parameter */
			print_usage(argc, argv);
			return 0;
		}
		default:
		{
			printf("Invalid option: %c\n", c);
			printf("Try '%s -h' for more information.\n", argv[0]);
			return -1;
		}
		}
	}

	if(fd < 0)
	{
		fd = OpenDev("/dev/ttyAP2");
		strcpy(dev_name, "/dev/ttyAP2");
	}

	if(fd > 0)
	{
		if(set_speed(fd, speed) != 0)
			return -1;
	}
	else
	{
		return -1;
	}

	if(set_com(fd, databit, stopbit, parity, flowcontrol) != 0)
	{
		printf("Set Com Error\n");
		return -1;
	}

	print_config(dev_name, speed, databit, stopbit, parity, flowcontrol);

	printf("\nStarting...\n");
	while(1)
	{
		memset(buf, 0, MAX_BUF_LEN);

		if((nread = read(fd, buf, MAX_BUF_LEN)) > 0)
		{
			rx_count += nread;
			printf("rx_count: %d, nread = %d\n", rx_count, nread);
			fflush(stdout);
		}
	}

	close(fd);

	return 0;
}
